
# This file was automatically generated by Dist::Zilla::Plugin::ModuleBuild v6.005.
use strict;
use warnings;

use Module::Build 0.28;


my %module_build_args = (
  "build_requires" => {
    "Module::Build" => "0.28"
  },
  "configure_requires" => {
    "ExtUtils::MakeMaker" => 0,
    "Module::Build" => "0.28"
  },
  "dist_abstract" => "REST Client Role",
  "dist_author" => [
    "Kaare Rasmussen <kaare at cpan dot org>"
  ],
  "dist_name" => "Role-REST-Client",
  "dist_version" => "0.20",
  "license" => "perl",
  "module_name" => "Role::REST::Client",
  "recommends" => {
    "Data::Serializer::JSON" => 0,
    "Data::Serializer::XML::Simple" => 0,
    "Data::Serializer::YAML" => 0,
    "HTTP::Thin" => 0,
    "JSON" => "2.00",
    "XML::Simple" => 0,
    "YAML" => 0
  },
  "recursive_test_files" => 1,
  "requires" => {
    "Carp" => 0,
    "Data::Serializer::Raw" => 0,
    "HTTP::Headers" => 0,
    "HTTP::Message" => 0,
    "HTTP::Response" => 0,
    "HTTP::Status" => 0,
    "HTTP::Thin" => 0,
    "HTTP::Tiny" => 0,
    "Moo" => "1.003000",
    "Moo::Role" => 0,
    "MooX::HandlesVia" => "0.001004",
    "Try::Tiny" => 0,
    "Type::Tiny" => "0.024",
    "Types::Standard" => 0,
    "URI::Escape::XS" => "0.11"
  },
  "test_requires" => {
    "Data::Serializer::JSON" => 0,
    "Data::Serializer::XML::Simple" => 0,
    "Data::Serializer::YAML" => 0,
    "File::Find" => 0,
    "File::Temp" => 0,
    "HTTP::Request" => 0,
    "HTTP::Thin" => 0,
    "JSON" => "2.00",
    "Moose" => 0,
    "Plack::Component" => 0,
    "Plack::Request" => 0,
    "Plack::Runner" => 0,
    "Test::Deep" => 0,
    "Test::Exception" => 0,
    "Test::More" => "0.88",
    "Test::TCP" => 0,
    "Test::Warnings" => 0,
    "XML::Simple" => 0,
    "YAML" => 0,
    "parent" => 0,
    "strict" => 0,
    "utf8" => 0,
    "warnings" => 0
  }
);


my %fallback_build_requires = (
  "Data::Serializer::JSON" => 0,
  "Data::Serializer::XML::Simple" => 0,
  "Data::Serializer::YAML" => 0,
  "File::Find" => 0,
  "File::Temp" => 0,
  "HTTP::Request" => 0,
  "HTTP::Thin" => 0,
  "JSON" => "2.00",
  "Module::Build" => "0.28",
  "Moose" => 0,
  "Plack::Component" => 0,
  "Plack::Request" => 0,
  "Plack::Runner" => 0,
  "Test::Deep" => 0,
  "Test::Exception" => 0,
  "Test::More" => "0.88",
  "Test::TCP" => 0,
  "Test::Warnings" => 0,
  "XML::Simple" => 0,
  "YAML" => 0,
  "parent" => 0,
  "strict" => 0,
  "utf8" => 0,
  "warnings" => 0
);


unless ( eval { Module::Build->VERSION(0.4004) } ) {
  delete $module_build_args{test_requires};
  $module_build_args{build_requires} = \%fallback_build_requires;
}

my $build = Module::Build->new(%module_build_args);


$build->create_build_script;
